
// (C) 2010 by Travis Smith See licensing info in Orbduino.pde

void RTCinit()
{
	pinMode(RTCRSTpin, OUTPUT); 
	digitalWrite(RTCRSTpin, LOW); 
	pinMode(RTCSCLKpin, OUTPUT); 
	digitalWrite(RTCSCLKpin, LOW); 
}


void SetTime(stcTime myTime) 
{
	//Clear the WP bit so we can set the time, etc
	WriteRTC(0x8E, 0x00); //ctl reg write
	
	//Turn on trickle charger: 2 diodes, 8k resistor from VCC (5v)
	//WriteRTC(0x90, 0xAB); //trickle charger write, TCS=1010, DS=10, RS=11
	
	WriteRTC(0x90, 0x5C); //trickle charger OFF!
	
	//Need to send all 8 registers to be valid
	digitalWrite(RTCRSTpin, HIGH); 
	ByteOut(0xBE); //clock burst register write
	ByteOut(INTtoBCD(myTime.tSeconds)); //clear ClockHalt bit(7) too
	ByteOut(INTtoBCD(myTime.tMinutes)); 
	ByteOut(0x80 | (myTime.tAMPM << 5) | INTtoBCD(myTime.tHours)); // set 12/24 bit(7) high (12 hour mode)
	ByteOut(INTtoBCD(myTime.tDate)); 
	ByteOut(INTtoBCD(myTime.tMonth)); 
	ByteOut(INTtoBCD(myTime.tDayofWeek)); 
	ByteOut(INTtoBCD(myTime.tYear)); 
	ByteOut(0x80); // Set WP bit 7
	digitalWrite(RTCRSTpin, LOW); 
}


void GetTime(stcTime *pmyTime) 
{
	int val;
	
	digitalWrite(RTCRSTpin, HIGH); 
	
	ByteOut(0xBF); //clock burst register read
	
	val = ByteIn();
	pmyTime->tSeconds = BCDtoINT(val & 0x7F);
	
	val = ByteIn();
	pmyTime->tMinutes = BCDtoINT(val & 0x7F);
	
	val = ByteIn();
	pmyTime->tHours = BCDtoINT(val & 0x1F);
	pmyTime->tAMPM = ((val & 0x20) >> 5); //PMval=1
	
	val = ByteIn();
	pmyTime->tDate = BCDtoINT(val & 0x3F);
	
	val = ByteIn();
	pmyTime->tMonth = BCDtoINT(val & 0x1F);
	
	val = ByteIn();
	pmyTime->tDayofWeek = (val & 0x07);
	
	val = ByteIn();
	pmyTime->tYear = BCDtoINT(val);
	
	digitalWrite(RTCRSTpin, LOW); 
}


/*
void GetDateTimeStr(char *cpBuf)
{	
	//Returns date and time (in 12 hour AM/PM format) to the supplied buffer
	stcTime myTime;

	GetTime(&myTime);

	sprintf(cpBuf, "%d:%02d:%02d %s, %s %02d/%02d/%04d", \
		myTime.tHours, myTime.tMinutes, myTime.tSeconds, strAMPM[myTime.tAMPM], \
		strDays[myTime.tDayofWeek - 1], myTime.tMonth, myTime.tDate, (2000 + myTime.tYear));

}


void GetDateTime24Str(char *cpBuf)
{	
	//Returns date and time (in 24 hour format) to the supplied buffer
	stcTime myTime;

	GetTime(&myTime);

	sprintf(cpBuf, "%04d/%02d/%02d %02d:%02d:%02d", \
		(2000 + myTime.tYear), myTime.tMonth, myTime.tDate, \
		(myTime.tHours == 12 ? (12 * myTime.tAMPM) : (myTime.tHours + 12 * myTime.tAMPM)), \
		myTime.tMinutes, myTime.tSeconds);
}
*/


int BCDtoINT(int BCDval)
{
	return ((BCDval & 0xF0) >> 4) * 10 + (BCDval & 0x0F);
}


int INTtoBCD(int INTval)
{
	int BCDval = 0;

	while (INTval >= 10) {
		INTval -= 10;
		BCDval += 0x10; 
	}

	BCDval |= INTval; 

	return (BCDval);
}


void  ByteOut(unsigned char ToSend)
{
	pinMode(RTCIOpin, OUTPUT); 
	shiftOut(RTCIOpin, RTCSCLKpin, LSBFIRST, ToSend);  
}

unsigned char  ByteIn()
{
	pinMode(RTCIOpin, INPUT); 
	unsigned char Result = 0;
	
	for (int Count = 1; Count < 256; Count *= 2) 
	{
		if ( digitalRead(RTCIOpin)) 
		{
			Result |= Count; 
		} 
		digitalWrite(RTCSCLKpin, HIGH); 
		digitalWrite(RTCSCLKpin, LOW); 
	}
	return Result;
}

void WriteRTC(unsigned char Addr, unsigned char Value)
{
	digitalWrite(RTCRSTpin, HIGH); 
	ByteOut((Addr & 0xFE)); //clear r/w* bit (0) 
	ByteOut(Value); 
	digitalWrite(RTCRSTpin, LOW); 
}

unsigned char ReadRTC(unsigned char Addr)
{
	unsigned char RetVal;
	
	digitalWrite(RTCRSTpin, HIGH); 
	ByteOut((Addr | 0x01)); //force read bit high (0)
	RetVal = ByteIn();
	digitalWrite(RTCRSTpin, LOW); 
	
	return(RetVal);
}


/******************************************************
 * Serial Functions
 *******************************************************/

void SerialPrintTime(stcTime myTime)
{
	SerialPrintPad(myTime.tHours);       
	Serial.print(':');   
	SerialPrintPad(myTime.tMinutes);       
	Serial.print(':');    
	SerialPrintPad(myTime.tSeconds); 
	Serial.print(' ');  
	Serial.print(strAMPM[myTime.tAMPM]);
	PgmPrint(", ");  
	Serial.print(strDays[myTime.tDayofWeek - 1]);       
	Serial.print(' ');    
	SerialPrintPad(myTime.tMonth);       
	Serial.print('/');    
	SerialPrintPad(myTime.tDate);       
	Serial.print('/');    
	Serial.println((2000 + myTime.tYear), DEC);       
}


void SerialPrintPad(char cNum)
{
	if (cNum < 10) Serial.print('0'); 
	Serial.print(cNum, DEC); 
}


boolean SerialInputTime(stcTime *pmyTime)
{
	char x, val;
	
	PgmPrintln("\nEnter Current Date/Time as dMMDDYYhhmmss");
	PgmPrintln("d=Day of week 1(Sun)-7(Sat), hh=24 hr clk");
	
	for (x=0; x<13; x++) 
	{
		if (!SerialAvailabeTimeout()) return(false);

		val = Serial.read();
		if (val < '0' || val > '9') return(false);
		val -= '0';
		
		switch (x) 
		{
			case 0:
				pmyTime->tDayofWeek = val;
				break;
			case 1:
				pmyTime->tMonth = val*10;
				break;
			case 2:
				pmyTime->tMonth += val;
				break;
			case 3:
				pmyTime->tDate = val*10;
				break;
			case 4:
				pmyTime->tDate += val;
				break;
			case 5:
				pmyTime->tYear = val*10;
				break;
			case 6:
				pmyTime->tYear += val;
				break;
			case 7:
				pmyTime->tHours = val*10;
				break;
			case 8:
				pmyTime->tHours += val;
				break;
			case 9:
				pmyTime->tMinutes = val*10;
				break;
			case 10:
				pmyTime->tMinutes += val;
				break;
			case 11:
				pmyTime->tSeconds = val*10;
				break;
			case 12:
				pmyTime->tSeconds += val;
				break;
		}
	}
	
	if (pmyTime->tHours >= 12) pmyTime->tAMPM = PMval;
	else pmyTime->tAMPM = AMval;
	
	if (pmyTime->tHours > 12) pmyTime->tHours -=12;

	return(true);
}  


bool SerialAvailabeTimeout()
{
	unsigned long ulStartMillis = millis();
	
	while(!Serial.available() && (millis() - ulStartMillis < SERIAL_INPUT_TIMEOUT_MS)); // timeout loop
	
	if (Serial.available())
	{
		return(true);
	}
	else
	{
		PgmPrintln("\nTimeout!");  
		return(false);
	}
}




